'' =================================================================================================
''
''   File....... jm_hc-05_configuration.spin
''   Purpose....
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (C) 2016 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started....
''   Updated.... 02 FEB 2016
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x                                     
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                         ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us


  TRM_BAUD = 115_200
  BT_BAUD  =  38_400 
  

con { io pins }

  RX1 = 31                                                      ' programming / terminal
  TX1 = 30

  SDA = 29                                                      ' eeprom / i2c
  SCL = 28

  LED = 26

  
  ' HC-05 Bluetooth connections

  ST  =  4 { I }                                                ' state
  KEY =  3 { O }                                                ' High = AT work mode
  RST =  2 { O/I }                                              ' Low = BT in reset mode                                                    ' 
  RX2 =  1 { I }                                                ' requires pull-up
  TX2 =  0 { O }                                                ' may need pull-up 


con

  #-1, ON, OFF                                                  ' digital control


obj

' main                                                          ' * master Spin cog
  time : "jm_time"                                              '   timing and delays
  io   : "jm_io_basic"                                          '   essential io
  buf  : "jm_textbuf"                                           '   text buffer
  term : "jm_fullduplexserial"                                  ' * serial IO for terminal
  hc05 : "jm_fullduplexserial"                                  ' * serial IO for hc-05     
                                                                 
' * uses cog when loaded                                         
                                                                 
                                                                 
var

                                                             
pub main | check                                                     
                                                                 
  setup

  time.pause(2000)                                              ' open PST now!

  term.tx(term#CLS)
  term.str(string("Ready to accept AT commands", 13, 13))

  repeat
    outa[LED] := ina[ST]                                        ' follow state output
    
    get_command
    send_command
    if (get_response)
      show_response(-1)
    else
      show_response(string("Timeout/Response Error"))


pub get_command | c

'' Allow simple editing of command before sending to HC-05

  buf.reset

  repeat
    c := term.rx
    case c
       8    : buf.truncate(buf.len-1)
      13    : return
      other : buf.putc(buf.upper(c))
  
 
pub send_command

  term.str(string("--> "))                                      ' display on terminal
  term.str(buf.address)
  term.tx(13)

  hc05.rxflush                                                  ' clean up incoming buffer
  hc05.str(buf.address)                                         ' send command 
  hc05.tx(13)
  hc05.tx(10)                                                   ' required by HC-05
  
 
pub get_response | c

  buf.reset

  c := hc05.rxtime(10000)
  if (c < 0)
    return false
  else
    buf.putc(c)

  repeat
    c := hc05.rxtime(25)
    if (c == 13)
      return true
    else
      buf.putc(c) 
    

pub show_response(p_str)

  if (p_str < 0)
    term.str(string("<-- "))
    term.str(buf.address)
    term.tx(13)
  else
    term.str(string("*** "))   
    term.str(p_str)            
    term.tx(13)
                                                                
  term.tx(13)                

                                                                 
pub setup                                                        
                                                                 
'' Setup IO and objects for application                          
                                                                 
  time.start                                                    ' setup timing & delays
                                                                 
  io.start(0, 0)                                                ' clear all pins (master cog)
  io.low(LED)
                                                                 
  term.start(RX1, TX1, %0000, TRM_BAUD)                         ' start serial for terminal *

  hc05.start(RX2, TX2, %0000,  BT_BAUD)                         ' start serial for hc-05 *

  
  ' put HC-05 into configuration mode
  ' -- all AT commands work in this mode
  
  io.low(RST)
  time.pause(500)
  io.high(KEY)
  time.pause(500)
  io.input(RST)

  term.rxflush
  hc05.rxflush  

   
dat { license }                                                  

{{

  Copyright (C) 2016 Jon McPhalen

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}