'' =================================================================================================
''
''   File....... jm_textbuf.spin
''   Purpose.... Text buffer object for manipulating strings
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (C) 2015-2016 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started....
''   Updated.... 23 JAN 2016
''
'' =================================================================================================


con

  BUF_SIZE = 80                                                 ' 80 chars 


var

  byte  buffer[BUF_SIZE+1]                                      ' buffer + terminating 0
  byte  bidx                                                    ' buffer index (next char)
                                                                 
                                                                 
pub null                                                         
                                                                 
  ' This is not a top-level object                               
                                                                 
                                                                 
pub reset                                                        
                                                                 
'' Clears buffer and resets buffer index                         

  bytefill(@buffer, 0, BUF_SIZE+2)                              ' clear buffer & bidx


pub putc(c)

'' Add character to end of buffer

  if (bidx < BUF_SIZE)                                          ' if space in buffer
    buffer[bidx++] := c                                         '  add new character
                                                                 
                                                                 
pub puts(p_str)                                                  
                                                                 
'' Add string to end of buffer                                   
                                                                 
  repeat strsize(p_str)                                         ' loop through p_str
    putc(byte[p_str++])                                         '   add char if space available        


pub copy(p_str) | n

'' Copies string at p_str to buffer
'' -- replaces anything already in buffer

  n := strsize(p_str)                                           ' number of bytes in source string
                                                                 
  if (n < BUF_SIZE)                                             ' will it fit?
    reset                                                       ' yes, clear buffer
    bytemove(@buffer, p_str, n)                                 ' copy new string into buffer
    bidx := n                                                   ' update buffer index
    return true                                                 ' report success
                                                                 
  return false


pub dec(value) : zflag | divisor, nx                            ' borrowed from FDS

'' Add decimal number to buffer

  nx := (value == negx)                                 
  if (value < 0)                                        
    value := ||(value+nx)                               
    putc("-")                                           
                                                        
  divisor := 1_000_000_000                              
                                                        
  repeat 10                                             
    if (value => divisor)                               
      putc(value / divisor + "0" + nx*(divisor == 1))    
      value //= divisor                                 
      zflag := true                                     
    elseif (zflag or (divisor == 1))                    
      putc("0")                                         
    divisor /= 10                                       


pub bin(value) : zflag | idx, c

'' Add variable-width binary value to buffer

  repeat idx from 31 to 0
    c := (value <-= 1) & 1 + "0"
    if (c <> "0")
      zflag := true
    if ((zflag) or (idx == 0))
      putc(c)


pub xbin(value, digits)                                         ' borrowed from FDS 

'' Add fixed-width binary number to buffer

  value <<= 32 - digits
  repeat digits
    putc((value <-= 1) & 1 + "0")


pub hex(value) : zflag | idx, c

'' Add variable-width hex value to buffer

  repeat idx from 7 to 0
    c := lookupz((value <-= 4) & $F : "0".."9", "A".."F")
    if (c <> "0")
      zflag := true
    if ((zflag) or (idx == 0))
      putc(c)
      

pub xhex(value, digits)                                         ' borrowed from FDS 

'' Add fixed-width hex number to buffer

  value <<= (8 - digits) << 2
  repeat digits
    putc(lookupz((value <-= 4) & $F : "0".."9", "A".."F"))


pub equal(p_str)

'' Return true if buffer is equal to string at p_str
'' -- case sensitive

  return strcomp(@buffer, p_str)


pub nc_equal(p_str) | tlen, idx

'' Return true if buffer is equal to string at p_str
'' -- not case sensitive

  tlen := strsize(p_str)                                        ' length of test string
                                                                 
  if (len <> tlen)                                              ' if not equal
    return false                                                '  no match
                                                                 
  repeat idx from 0 to tlen-1                                    
    if (upper(buffer[idx]) <> upper(byte[p_str][idx]))          ' test both as upper case
      return false

  return true
    

pub len

'' Returns length of buffer (as string)

  return strsize(@buffer)


pub upper(c)

'' Convert c to upper case

  if ((c => "a") and (c =< "z"))
    c -= 32

  return c
  

pub ucase | idx

'' Converts alpha characters in buffer to upper case
                    
  repeat idx from 0 to bidx
    buffer[idx] := upper(buffer[idx])


pub lower(c)

'' Convert c to lower case

  if ((c => "A") and (c =< "Z"))
    c += 32

  return c

    
pub lcase | idx

'' Converts alpha characters in buffer to lower case
                    
  repeat idx from 0 to bidx
    if ((buffer[idx] => "A") and (buffer[idx] =< "Z"))
      buffer[idx] := lower(buffer[idx])  


pub truncate(n)

'' Sets index of buffer to n
'' -- only if buffer longer than n

  if ((n => 0) and (n < bidx))
    bidx := n    
    bytefill(@buffer[n], 0, BUF_SIZE+1-n)
    return true

  return false

      
pub address

'' Provides buffer address for external string methods

  return @buffer
   
  
dat { license }

{{

  Copyright (C) 2015-2016 Jon McPhalen 

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}